import { __decorate, __rest } from "tslib";
import { ClientSideRowModelModule } from '@ag-grid-community/client-side-row-model';
import { Grid, ComponentUtil, Events, ModuleRegistry, } from '@ag-grid-community/core';
import { CsvExportModule } from '@ag-grid-community/csv-export';
import { Session } from '@genesislcap/foundation-comms';
import { insertDocumentCSSRule, LifecycleMixin } from '@genesislcap/foundation-utils';
import { attr, DOM, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { classNames } from '@microsoft/fast-web-utilities';
import { GridProCell } from './cell';
import { ActionRenderer, ActionsMenuRenderer, BooleanRenderer } from './cell-renderers';
import { GridColumn } from './column';
import { agThemeFontFaceMap, defaultAgGridFontFace } from './external';
import { GridProGenesisDatasource, GridProGenesisDatasourceNext, } from './grid-pro-genesis-datasource';
import { foundationGridProStyles as styles } from './grid-pro.styles';
import { gridProTemplate as template } from './grid-pro.template';
import { AgGridTheme, GridProRendererTypes, agThemeFontsId, agThemeTokenMapClassname, } from './grid-pro.types';
GridProGenesisDatasource;
GridProGenesisDatasourceNext;
GridProCell;
GridColumn;
ModuleRegistry.registerModules([CsvExportModule, ClientSideRowModelModule]);
const hypenateAndLowercase = (property) => property.replace('/([a-z])([A-Z])/g', '$1-$2').toLowerCase();
export class GridPro extends LifecycleMixin(FoundationElement) {
    themeChanged(oldValue, newValue) {
        if (!newValue) {
            this.classNames = '';
            this.destroyFontFace();
            return;
        }
        this.classNames = classNames(agThemeTokenMapClassname(newValue), newValue);
        this.insertFontFace(newValue);
    }
    constructor() {
        super();
        this.autoCellRendererByType = false;
        this.onlyTemplateColDefs = false;
        this.enabledRowFlashing = false;
        this.enableCellFlashing = false;
        this.gridFontFace = defaultAgGridFontFace;
        this.columnComponentName = 'grid-pro-column';
        this.theme = AgGridTheme.alpine;
        this.rehydrationAttempted = false;
        this.agAttributes = {};
        this.agPropertiesMap = ComponentUtil.ALL_PROPERTIES.reduce((map, property) => {
            map[property.toLowerCase()] = property;
            map[hypenateAndLowercase(property)] = property;
            return map;
        }, {});
    }
    connectedCallback() {
        super.connectedCallback();
        this.rehydrationAttempted = false;
        if (!this.shouldRunConnect)
            return;
        ComponentUtil.ALL_PROPERTIES.forEach((key) => {
            Object.defineProperty(this, key, {
                set: (v) => {
                    this.agAttributeChangedCallback(key.toLowerCase(), undefined, v);
                },
                enumerable: true,
                configurable: true,
            });
        });
    }
    disconnectedCallback() {
        var _a;
        super.disconnectedCallback();
        this.rehydrationAttempted = false;
        if (!this.shouldRunDisconnect)
            return;
        (_a = this.gridApi) === null || _a === void 0 ? void 0 : _a.destroy();
    }
    combineAllGridComponents(gridOptionsComponents) {
        const defaultFoundationAgComponents = {
            [GridProRendererTypes.action]: ActionRenderer,
            [GridProRendererTypes.actionsMenu]: ActionsMenuRenderer,
            [GridProRendererTypes.boolean]: BooleanRenderer,
        };
        return Object.assign(Object.assign(Object.assign({}, defaultFoundationAgComponents), gridOptionsComponents), this.gridComponents);
    }
    statePersistanceEnabled() {
        if (!this.persistColumnStateKey || this.persistColumnStateKey.length === 0) {
            return false;
        }
        return true;
    }
    serialiseStoreTableConfig() {
        if (this.rehydrationAttempted && this.statePersistanceEnabled()) {
            const existingOptions = JSON.parse(this.session.getLocalStorageItem('gridOptions') || '{}');
            const colState = this.agGridOptions.columnApi.getColumnState();
            existingOptions[this.persistColumnStateKey] = JSON.stringify(colState);
            this.session.setLocalStorageItem('gridOptions', JSON.stringify(existingOptions));
        }
    }
    rehydrateTableConfig() {
        this.rehydrationAttempted = true;
        if (this.statePersistanceEnabled() && this.session.getLocalStorageItem('gridOptions')) {
            const allGridOptions = JSON.parse(this.session.getLocalStorageItem('gridOptions'));
            if (this.persistColumnStateKey in allGridOptions) {
                this.agGridOptions.columnApi.applyColumnState({
                    state: JSON.parse(allGridOptions[this.persistColumnStateKey]),
                    applyOrder: true,
                });
            }
        }
    }
    set gridOptions(options) {
        const globalEventListener = this.globalEventListener.bind(this);
        const serialiseStoreTableConfig = this.serialiseStoreTableConfig.bind(this);
        const rehydrateTableConfig = this.rehydrateTableConfig.bind(this);
        Array.from(this.attributes).forEach((attribute) => {
            const attrName = this.agPropertiesMap[attribute.name];
            if (!attrName)
                return;
            this.agAttributes[attrName] = attribute.value;
        });
        const { columnDefs, components, onGridReady } = options, rest = __rest(options, ["columnDefs", "components", "onGridReady"]);
        const derivedOptions = Object.assign(Object.assign(Object.assign({ defaultColDef: {
                filter: true,
                resizable: true,
                sortable: true,
            }, columnDefs: this.applyTemplateDefinitions(columnDefs), components: this.combineAllGridComponents(components), suppressDragLeaveHidesColumns: true }, this.eventsAndCallbacks), { onGridReady: (params) => {
                var _a, _b;
                this.gridApi = (_a = options.api) !== null && _a !== void 0 ? _a : params.api;
                this.columnApi = (_b = options.columnApi) !== null && _b !== void 0 ? _b : params.columnApi;
                if (onGridReady) {
                    onGridReady(params);
                }
                this.$emit('onGridReady', {
                    params,
                    gridId: this.id,
                });
                rehydrateTableConfig();
            }, onFirstDataRendered: () => {
                this.enableFlashingRows.bind(this);
            }, onColumnPinned: serialiseStoreTableConfig, onColumnResized: serialiseStoreTableConfig, onColumnMoved: serialiseStoreTableConfig, onDisplayedColumnsChanged: serialiseStoreTableConfig, onFilterChanged: serialiseStoreTableConfig, onGridSizeChanged: serialiseStoreTableConfig, onSortChanged: serialiseStoreTableConfig }), rest);
        this.agGridOptions = Object.assign(Object.assign({}, this.agGridOptions), ComponentUtil.copyAttributesToGridOptions(derivedOptions, this.agAttributes));
        const gridParams = {
            globalEventListener: globalEventListener,
        };
        DOM.queueUpdate(() => {
            if (this.initialised) {
                if (this.gridSlot.firstChild) {
                    this.gridSlot.removeChild(this.gridSlot.firstChild);
                }
            }
            this.agGrid = new Grid(this.gridSlot, this.agGridOptions, gridParams);
            this.initialised = true;
        });
    }
    enableFlashingRows() {
        if (this.enabledRowFlashing) {
            this.gridApi.addEventListener(Events.EVENT_ASYNC_TRANSACTIONS_FLUSHED, (e) => {
                e.results.forEach((result) => {
                    if (result.add.length) {
                        this.gridApi.flashCells({
                            rowNodes: result.add,
                        });
                    }
                });
            });
        }
    }
    applyTemplateDefinitions(columnDefs = []) {
        const agColumnNodes = this.querySelectorAll(this.columnComponentName);
        const overrides = Array.from(agColumnNodes).map((x) => { var _a; return (_a = x.definition) === null || _a === void 0 ? void 0 : _a.field; });
        if (agColumnNodes.length === 0)
            return columnDefs;
        const mergedColumnDefs = [
            ...(columnDefs ? columnDefs.filter((g) => !overrides.includes(g.field)) : []),
            ...Array.from(agColumnNodes).map((x) => {
                var _a;
                if (this.enableCellFlashing && x.definition.enableCellChangeFlash === undefined) {
                    x.definition.enableCellChangeFlash = true;
                }
                if (!this.autoCellRendererByType || x.definition.valueFormatter) {
                    return x.definition;
                }
                const typeValueFormatter = (_a = columnDefs === null || columnDefs === void 0 ? void 0 : columnDefs.find((colDef) => x.definition.field === colDef.field)) === null || _a === void 0 ? void 0 : _a.valueFormatter;
                if (typeValueFormatter) {
                    x.definition.valueFormatter = typeValueFormatter;
                }
                return x.definition;
            }),
        ];
        return mergedColumnDefs;
    }
    get observedAttributes() {
        const lowerCasedPropertyNames = ComponentUtil.ALL_PROPERTIES.map((property) => property.toLowerCase());
        const hyphenatedPropertyNames = ComponentUtil.ALL_PROPERTIES.map((property) => hypenateAndLowercase(property));
        return lowerCasedPropertyNames.concat(hyphenatedPropertyNames);
    }
    agAttributeChangedCallback(attName, oldValue, newValue) {
        if (oldValue === newValue) {
            return;
        }
        const gridPropertyName = this.agPropertiesMap[attName];
        if (gridPropertyName) {
            this.agAttributes[gridPropertyName] = newValue;
            if (this.initialised) {
                const changeObject = {};
                changeObject[gridPropertyName] = { currentValue: newValue };
                ComponentUtil.processOnChange(changeObject, this.gridApi);
            }
        }
    }
    globalEventListener(eventType, event) {
        const eventLowerCase = eventType.toLowerCase();
        const browserEvent = new Event(eventLowerCase);
        const browserEventNoType = browserEvent;
        browserEventNoType.agGridDetails = event;
        this.dispatchEvent(browserEvent);
        const callbackMethod = 'on' + eventLowerCase;
        if (typeof this[callbackMethod] === 'function') {
            this[callbackMethod](browserEvent);
        }
    }
    insertFontFace(newValue) {
        const fontFaceRule = agThemeFontFaceMap[newValue] || this.gridFontFace;
        this.destroyFontFace();
        const newDestroy = insertDocumentCSSRule(fontFaceRule, agThemeFontsId(newValue));
        if (newDestroy) {
            this.fontFaceDestroy = newDestroy;
        }
    }
    destroyFontFace() {
        if (this.fontFaceDestroy) {
            this.fontFaceDestroy();
            this.fontFaceDestroy = null;
        }
    }
}
__decorate([
    observable
], GridPro.prototype, "columnApi", void 0);
__decorate([
    observable
], GridPro.prototype, "gridApi", void 0);
__decorate([
    Session
], GridPro.prototype, "session", void 0);
__decorate([
    attr({ attribute: 'auto-cell-renderer-by-type', mode: 'boolean' })
], GridPro.prototype, "autoCellRendererByType", void 0);
__decorate([
    attr({ attribute: 'only-template-col-defs', mode: 'boolean' })
], GridPro.prototype, "onlyTemplateColDefs", void 0);
__decorate([
    attr({ mode: 'boolean' })
], GridPro.prototype, "enabledRowFlashing", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'enable-cell-flashing' })
], GridPro.prototype, "enableCellFlashing", void 0);
__decorate([
    attr({ attribute: 'persist-column-state-key' })
], GridPro.prototype, "persistColumnStateKey", void 0);
__decorate([
    observable
], GridPro.prototype, "gridFontFace", void 0);
__decorate([
    attr
], GridPro.prototype, "columnComponentName", void 0);
__decorate([
    observable
], GridPro.prototype, "eventsAndCallbacks", void 0);
__decorate([
    attr
], GridPro.prototype, "headerHeight", void 0);
__decorate([
    attr
], GridPro.prototype, "rowHeight", void 0);
__decorate([
    attr
], GridPro.prototype, "theme", void 0);
__decorate([
    observable
], GridPro.prototype, "classNames", void 0);
__decorate([
    observable
], GridPro.prototype, "agGrid", void 0);
__decorate([
    observable
], GridPro.prototype, "gridSlot", void 0);
__decorate([
    observable
], GridPro.prototype, "gridComponents", void 0);
export const foundationGridProShadowOptions = undefined;
export const defaultGridProConfig = {
    shadowOptions: undefined,
};
export const foundationGridPro = GridPro.compose(Object.assign({ baseName: 'grid-pro', styles,
    template, shadowOptions: foundationGridProShadowOptions }, defaultGridProConfig));
